@@ -13,11 +13,9 @@ import collections
 from datetime import timedelta
 import re
 from functools import update_wrapper
-
 from flask import make_response, request, current_app
 from six import string_types
 
-
 # Common string constants
 ACL_ORIGIN = 'Access-Control-Allow-Origin'
 ACL_METHODS = 'Access-Control-Allow-Methods'
@@ -25,24 +23,26 @@ ACL_HEADERS = 'Access-Control-Allow-Headers'
 ACL_EXPOSE_HEADERS = 'Access-Control-Expose-Headers'
 ACL_CREDENTIALS = 'Access-Control-Allow-Credentials'
 ACL_MAX_AGE = 'Access-Control-Max-Age'
-
+ACL_REQUEST_METHOD = 'Access-Control-Request-Method'
+ACL_REQUEST_HEADERS = 'Access-Control-Request-Headers'
 
 ALL_METHODS = ['GET', 'HEAD', 'POST', 'OPTIONS', 'PUT', 'PATCH', 'DELETE']
 
 CONFIG_OPTIONS = ['CORS_ORIGINS', 'CORS_METHODS', 'CORS_HEADERS',
                   'CORS_EXPOSE_HEADERS', 'CORS_SUPPORTS_CREDENTIALS',
                   'CORS_MAX_AGE', 'CORS_SEND_WILDCARD', 'CORS_ALWAYS_SEND',
-                  'CORS_AUTOMATIC_OPTIONS', 'CORS_VARY_HEADER']
+                  'CORS_AUTOMATIC_OPTIONS', 'CORS_VARY_HEADER',
+                  'CORS_RESOURCES', 'CORS_INTERCEPT_EXCEPTIONS']
 
 FLASK_CORS_EVALUATED = '_FLASK_CORS_EVALUATED'
 
 _defaults_dict = dict(origins='*',
+                      methods=ALL_METHODS,
                       always_send=True,
                       automatic_options=True,
                       send_wildcard=True,
                       vary_header=True)
 
-
 def cross_origin(*args, **kwargs):
     '''
     This function is the decorator which is used to wrap a Flask route with.
@@ -61,23 +61,23 @@ def cross_origin(*args, **kwargs):
     :type origins: list or string
 
     :param methods: The method or list of methods which the allowed origins
-        are allowed to access.
+        are allowed to access for non-simple requests.
 
         Default : [GET, HEAD, POST, OPTIONS, PUT, PATCH, DELETE]
     :type methods: list or string
 
-    :param headers: The header or list of header field names which can be used
-        when this resource is accessed by allowed origins.
-
-        Default : None
-    :type headers: list or string
-
-    :param expose_headers: The header or list of headers which are are safe to
-        expose to browsers.
+    :param expose_headers: The header or list which are safe to expose to the
+        API of a CORS API specification
 
         Default : None
     :type expose_headers: list or string
 
+    :param headers: The header or list of header field names which can be used
+        when this resource is accessed by allowed origins
+
+        Default : None
+    :type headers: list or string
+
     :param supports_credentials: Allows users to make authenticated requests.
         If true, injects the `Access-Control-Allow-Credentials` header in
         responses.
@@ -218,15 +218,13 @@ class CORS(object):
         options.update(_get_app_kwarg_dict(app))
         options.update(kwargs)
 
-        _kwarg_resources = kwargs.get('resources')
-        _app_resources = app.config.get('CORS_RESOURCES')
-        _resources = _kwarg_resources or _app_resources or r'*'
+        _resources = options.get('resources', [r'/*'])
+        _intercept_exceptions = options.get('intercept_exceptions', True)
 
         if isinstance(_resources, dict):  # sort the regexps by length
             # To make the API more consistent with the decorator, allow a
             # resource of '*', which is not actually a valid regexp.
-            _resources = map(lambda x: (_re_fix(x[0]), x[1]),
-                             _resources.items())
+            _resources = [(_re_fix(k), v) for k, v in _resources.items()]
 
             # Sort by regex length to provide consistency of matching and
             # to provide a proxy for specificity of match. E.G. longer
@@ -261,6 +259,88 @@ class CORS(object):
 
         app.after_request(cors_after_request)
 
+        # Wrap exception handlers with cross_origin
+        # These error handlers will still respect the behavior of the route
+        if _intercept_exceptions:
+            def _after_request_decorator(f):
+                def wrapped_function(*args, **kwargs):
+                    return cors_after_request(app.make_response(f(*args, **kwargs)))
+                return wrapped_function
+
+            app.handle_exception = _after_request_decorator(
+                app.handle_exception)
+            app.handle_user_exception = _after_request_decorator(
+                app.handle_user_exception)
+
+
+def _get_cors_origin(options, request_origin):
+    origins = options.get('origins')
+    wildcard = '*' in origins
+    # If the Origin header is not present terminate this set of steps.
+    # The request is outside the scope of this specification.-- W3Spec
+    if request_origin:
+
+        # If the allowed origins is an asterisk or 'wildcard', always match
+        if wildcard:
+            if options.get('send_wildcard'):
+                return '*'
+            else:
+                return request_origin
+
+        # If the value of the Origin header is a case-sensitive match
+        # for any of the values in list of origins
+        elif any(_try_match(pattern, request_origin) for pattern in origins):
+            # Add a single Access-Control-Allow-Origin header, with either
+            # the value of the Origin header or the string "*" as value.
+            # -- W3Spec
+            return request_origin
+        else:
+            return None
+
+    # Unless always_send is set, then ignore W3 spec as long as there is a
+    # valid list of origins, e.g. one that is not merely comrpised of regular
+    # expressions.
+    # TODO: remove this. It breaks the spec, and practically shouldn't be used
+    # This will only be exceuted it there is no Origin in the request, in which
+    # case, why bother with CORS?
+    elif options.get('always_send') and options.get('origins_str'):
+        return options.get('origins_str')
+    # Terminate these steps, return the original request untouched.
+    else:
+        return None
+
+
+def _get_cors_headers(options, request_headers, request_method, response_headers=None):
+    headers = dict(response_headers or {})  # copy dict
+    origin_to_set = _get_cors_origin(options, request_headers.get('Origin'))
+
+    if origin_to_set is None:  # CORS is not enabled for this route
+        return headers
+
+    headers[ACL_ORIGIN] = origin_to_set
+    headers[ACL_HEADERS] = options.get('headers')
+
+    if options.get('supports_credentials'):
+        headers[ACL_CREDENTIALS] = 'true' # case sensative
+
+    # This is a preflight request
+    # http://www.w3.org/TR/cors/#resource-preflight-requests
+    if request_method == 'OPTIONS':
+        acl_request_method = request_headers.get(ACL_REQUEST_METHOD, '').upper()
+        # If there is no Access-Control-Request-Method header or if parsing
+        # failed, do not set any additional headers
+        if acl_request_method and acl_request_method in options.get('methods'):
+            headers[ACL_MAX_AGE] = options.get('max_age')
+            headers[ACL_METHODS] = options.get('methods')
+            headers[ACL_EXPOSE_HEADERS] = options.get('expose_headers')
+
+    # http://www.w3.org/TR/cors/#resource-implementation
+    if headers[ACL_ORIGIN] != '*' and options.get('vary_header'):
+        vary = ['Origin', headers.get('Vary', None)]
+        headers['Vary'] = ', '. join(v for v in vary if v is not None)
+
+    return dict((k, v) for k, v in headers.items() if v is not None)
+
 
 def _set_cors_headers(resp, options):
     '''
@@ -275,74 +355,27 @@ def _set_cors_headers(resp, options):
     if hasattr(resp, FLASK_CORS_EVALUATED):
         return resp
 
-    request_origin = request.headers.get('Origin', None)
-    origins = options.get('origins')
-    wildcard = '*' in origins
-    # If the Origin header is not present terminate this set of steps.
-    # The request is outside the scope of this specification.-- W3Spec
-    if request_origin:
+    headers_to_set = _get_cors_headers(options,
+                                       request.headers,
+                                       request.method,
+                                       resp.headers)
 
-        # If the allowed origins is an asterisk or 'wildcard', always match
-        if wildcard:
-            if options.get('send_wildcard'):
-                resp.headers[ACL_ORIGIN] = '*'
-            else:
-                resp.headers[ACL_ORIGIN] = request_origin
+    for k, v in headers_to_set.items():
+        resp.headers[k] = v
 
-        # If the value of the Origin header is a case-sensitive match
-        # for any of the values in list of origins
-        elif any(_try_match(pattern, request_origin) for pattern in origins):
-            # Add a single Access-Control-Allow-Origin header, with either
-            # the value of the Origin header or the string "*" as value.
-            # -- W3Spec
-            resp.headers[ACL_ORIGIN] = request_origin
-        else:
-            return resp
-
-    # Unless always_send is set, then ignore W3 spec as long as there is a
-    # valid list of origins, e.g. one that is not merely comrpised of regular
-    # expressions.
-    elif options.get('always_send') and options.get('origins_str'):
-        resp.headers[ACL_ORIGIN] = options.get('origins_str')
-    # Terminate these steps, return the original request untouched.
-    else:
-        return resp
-
-    if options.get('methods'):
-        resp.headers[ACL_METHODS] = options.get('methods')
-
-    if options.get('headers'):
-        resp.headers[ACL_HEADERS] = options.get('headers')
-
-    if options.get('expose_headers'):
-        resp.headers[ACL_EXPOSE_HEADERS] = options.get(
-            'expose_headers')
-
-    if options.get('max_age'):
-        resp.headers[ACL_MAX_AGE] = options.get('max_age')
-
-    if options.get('supports_credentials'):
-        resp.headers[ACL_CREDENTIALS] = 'true'
-
-    if request.method == 'OPTIONS':
-        resp.headers[ACL_METHODS] = options.get('methods',
-                                                _flexible_str(ALL_METHODS))
-
-    # http://www.w3.org/TR/cors/#resource-implementation
-    if resp.headers[ACL_ORIGIN] != '*' and options.get('vary_header'):
-        vary = ['Origin', resp.headers.get('Vary', None)]
-        resp.headers['Vary'] = ', '. join(v for v in vary if v is not None)
+    return resp
 
 
-def _get_app_kwarg_dict(app=current_app):
+def _get_app_kwarg_dict(appInstance=None):
     '''
         Returns the dictionary of CORS specific app configurations.
     '''
-    return dict([
-                (k.lower().replace('cors_', ''), app.config.get(k))
-                for k in CONFIG_OPTIONS
-                if app.config.get(k) is not None
-                ])
+    app = (appInstance or current_app)
+    return dict(
+        (k.lower().replace('cors_', ''), app.config.get(k))
+        for k in CONFIG_OPTIONS
+        if app.config.get(k) is not None
+    )
 
 
 def _re_fix(reg):
@@ -424,6 +457,7 @@ def _serialize_options(options):
     _serialize_option(options, 'methods', upper=True)
     _serialize_option(options, 'headers')
     _serialize_option(options, 'expose_headers')
+    _serialize_option(options, 'supports_credentials')
 
     if isinstance(options.get('max_age'), timedelta):
         options['max_age'] = str(int(options['max_age'].total_seconds()))
